@@ -14,53 +14,15 @@
 #include <dirent.h>
 #include <utime.h>
 
-#ifndef _WIN32
+#ifdef _WIN32
+  #include <direct.h>
+#else
   #include <poll.h>
   #include <sys/ioctl.h>
 #endif
 
 const char* FILE_NAME = "Sys/IO.c";
 
-int32_t SPVM__Sys__IO__rmdir(SPVM_ENV* env, SPVM_VALUE* stack) {
-  
-  void* obj_path = stack[0].oval;
-  
-  if (!obj_path) {
-    return env->die(env, stack, "The path must be defined", FILE_NAME, __LINE__);
-  }
-  
-  const char* path = env->get_chars(env, stack, obj_path);
-  int32_t status = rmdir(path);
-  if (status == -1) {
-    env->die(env, stack, "[System Error]rmdir failed:%s.", env->strerror(env, stack, errno, 0), FILE_NAME, __LINE__);
-    return SPVM_NATIVE_C_CLASS_ID_ERROR_SYSTEM;
-  }
-  
-  stack[0].ival = status;
-  
-  return 0;
-}
-
-int32_t SPVM__Sys__IO__unlink(SPVM_ENV* env, SPVM_VALUE* stack) {
-  
-  void* obj_path = stack[0].oval;
-
-  if (!obj_path) {
-    return env->die(env, stack, "The path must be defined", FILE_NAME, __LINE__);
-  }
-  
-  const char* path = env->get_chars(env, stack, obj_path);
-  int32_t status = unlink(path);
-  if (status == -1) {
-    env->die(env, stack, "[System Error]unlink failed:%s.", env->strerror(env, stack, errno, 0), FILE_NAME, __LINE__);
-    return SPVM_NATIVE_C_CLASS_ID_ERROR_SYSTEM;
-  }
-
-  stack[0].ival = status;
-  
-  return 0;
-}
-
 int32_t SPVM__Sys__IO__open(SPVM_ENV* env, SPVM_VALUE* stack) {
   
   int32_t items = env->get_args_stack_length(env, stack);
@@ -91,6 +53,102 @@ int32_t SPVM__Sys__IO__open(SPVM_ENV* env, SPVM_VALUE* stack) {
   return 0;
 }
 
+int32_t SPVM__Sys__IO__read(SPVM_ENV* env, SPVM_VALUE* stack) {
+  
+  int32_t e = 0;
+  
+  int32_t fd = stack[0].ival;
+  
+  void* obj_buffer = stack[1].oval;
+  
+  if (!obj_buffer) {
+    return env->die(env, stack, "The buffer must be defined", FILE_NAME, __LINE__);
+  }
+  
+  char* buffer = (char*)env->get_chars(env, stack, obj_buffer);
+  int32_t buffer_length = env->length(env, stack, obj_buffer);
+
+  int32_t count = stack[2].ival;
+  
+  if (!(count >= 0)) {
+    return env->die(env, stack, "The count must be more than or equal to 0", FILE_NAME, __LINE__);
+  }
+  
+  if (!(count <= buffer_length)) {
+    return env->die(env, stack, "The count must be less than the length of the buffer", FILE_NAME, __LINE__);
+  }
+  
+  int32_t read_length = read(fd, buffer, count);
+  if (read_length == -1) {
+    env->die(env, stack, "[System Error]read failed:%s.", env->strerror(env, stack, errno, 0), FILE_NAME, __LINE__);
+    return SPVM_NATIVE_C_CLASS_ID_ERROR_SYSTEM;
+  }
+  
+  stack[0].ival = read_length;
+  
+  return 0;
+}
+
+int32_t SPVM__Sys__IO__write(SPVM_ENV* env, SPVM_VALUE* stack) {
+  
+  int32_t e = 0;
+  
+  int32_t fd = stack[0].ival;
+  
+  void* obj_buffer = stack[1].oval;
+  
+  if (!obj_buffer) {
+    return env->die(env, stack, "The buffer must be defined", FILE_NAME, __LINE__);
+  }
+  
+  char* buffer = (char*)env->get_chars(env, stack, obj_buffer);
+  int32_t buffer_length = env->length(env, stack, obj_buffer);
+
+  int32_t count = stack[2].ival;
+  
+  if (!(count >= 0)) {
+    return env->die(env, stack, "The count must be more than or equal to 0", FILE_NAME, __LINE__);
+  }
+  
+  if (!(count <= buffer_length)) {
+    return env->die(env, stack, "The count must be less than the length of the buffer", FILE_NAME, __LINE__);
+  }
+  
+  int32_t write_length = write(fd, buffer, count);
+  if (write_length == -1) {
+    env->die(env, stack, "[System Error]write failed:%s.", env->strerror(env, stack, errno, 0), FILE_NAME, __LINE__);
+    return SPVM_NATIVE_C_CLASS_ID_ERROR_SYSTEM;
+  }
+  
+  stack[0].ival = write_length;
+  
+  return 0;
+}
+
+int32_t SPVM__Sys__IO__lseek(SPVM_ENV* env, SPVM_VALUE* stack) {
+  
+  int32_t fd = stack[0].ival;
+  
+  int64_t offset = stack[1].lval;
+  
+  if (!(offset >= 0)) {
+    return env->die(env, stack, "The offset must be greater than or equal to 0", FILE_NAME, __LINE__);
+  }
+
+  int32_t whence = stack[2].ival;
+
+  int64_t cur_offset = lseek(fd, offset, whence);
+  if (cur_offset == -1) {
+    env->die(env, stack, "[System Error]lseek failed:%s.", env->strerror(env, stack, errno, 0), FILE_NAME, __LINE__);
+    return SPVM_NATIVE_C_CLASS_ID_ERROR_SYSTEM;
+  }
+  
+  
+  stack[0].lval = cur_offset;
+  
+  return 0;
+}
+
 int32_t SPVM__Sys__IO__close(SPVM_ENV* env, SPVM_VALUE* stack) {
   
   int32_t fd = stack[0].ival;
@@ -106,77 +164,6 @@ int32_t SPVM__Sys__IO__close(SPVM_ENV* env, SPVM_VALUE* stack) {
   return 0;
 }
 
-int32_t SPVM__Sys__IO__clearerr(SPVM_ENV* env, SPVM_VALUE* stack) {
-  
-  void* obj_stream = stack[0].oval;
-  
-  if (!obj_stream) {
-    return env->die(env, stack, "The file stream must be defined", FILE_NAME, __LINE__);
-  }
-  
-  FILE* stream = env->get_pointer(env, stack, obj_stream);
-  
-  clearerr(stream);
-  
-  return 0;
-}
-
-int32_t SPVM__Sys__IO__feof(SPVM_ENV* env, SPVM_VALUE* stack) {
-  
-  void* obj_stream = stack[0].oval;
-  
-  if (!obj_stream) {
-    return env->die(env, stack, "The file stream must be defined", FILE_NAME, __LINE__);
-  }
-  
-  FILE* stream = env->get_pointer(env, stack, obj_stream);
-  
-  int32_t ret = feof(stream);
-  
-  stack[0].ival = ret;
-  
-  return 0;
-}
-
-int32_t SPVM__Sys__IO__ferror(SPVM_ENV* env, SPVM_VALUE* stack) {
-  
-  void* obj_stream = stack[0].oval;
-  
-  if (!obj_stream) {
-    return env->die(env, stack, "The file stream must be defined", FILE_NAME, __LINE__);
-  }
-  
-  FILE* stream = env->get_pointer(env, stack, obj_stream);
-  
-  int32_t ret = ferror(stream);
-  
-  stack[0].ival = ret;
-  
-  return 0;
-}
-
-int32_t SPVM__Sys__IO__fileno(SPVM_ENV* env, SPVM_VALUE* stack) {
-  
-  void* obj_stream = stack[0].oval;
-  
-  if (!obj_stream) {
-    return env->die(env, stack, "The file stream must be defined", FILE_NAME, __LINE__);
-  }
-  
-  FILE* stream = env->get_pointer(env, stack, obj_stream);
-  
-  int32_t fd = fileno(stream);
-
-  if (fd == -1) {
-    env->die(env, stack, "[System Error]fileno failed:%s.", env->strerror(env, stack, errno, 0), FILE_NAME, __LINE__);
-    return SPVM_NATIVE_C_CLASS_ID_ERROR_SYSTEM;
-  }
-  
-  stack[0].ival = fd;
-  
-  return 0;
-}
-
 static const int FILE_STREAM_CLOSED_INDEX = 0;
 
 int32_t SPVM__Sys__IO__fopen(SPVM_ENV* env, SPVM_VALUE* stack) {
@@ -242,39 +229,9 @@ int32_t SPVM__Sys__IO__fdopen(SPVM_ENV* env, SPVM_VALUE* stack) {
   return 0;
 }
 
-int32_t SPVM__Sys__IO__fread(SPVM_ENV* env, SPVM_VALUE* stack) {
-
-  int32_t e = 0;
+int32_t SPVM__Sys__IO__fileno(SPVM_ENV* env, SPVM_VALUE* stack) {
   
-  void* obj_buffer = stack[0].oval;
-  
-  if (!obj_buffer) {
-    return env->die(env, stack, "The buffer must be defined", FILE_NAME, __LINE__);
-  }
-  
-  char* buffer = (char*)env->get_chars(env, stack, obj_buffer);
-  int32_t buffer_length = env->length(env, stack, obj_buffer);
-  
-  int32_t size = stack[1].ival;
-  if (!(size >= 0)) {
-    return env->die(env, stack, "The size must be more than or equal to 0", FILE_NAME, __LINE__);
-  }
-  
-  int32_t data_length = stack[2].ival;
-  if (!(data_length >= 0)) {
-    return env->die(env, stack, "The data length must be more than or equal to 0", FILE_NAME, __LINE__);
-  }
-  
-  if (size == 0 || data_length == 0) {
-    stack[0].ival = 0;
-    return 0;
-  }
-  
-  if (!((buffer_length / size) <= data_length)) {
-    return env->die(env, stack, "The buffer length / the size must be less than or equal to the data length", FILE_NAME, __LINE__);
-  }
-  
-  void* obj_stream = stack[3].oval;
+  void* obj_stream = stack[0].oval;
   
   if (!obj_stream) {
     return env->die(env, stack, "The file stream must be defined", FILE_NAME, __LINE__);
@@ -282,43 +239,48 @@ int32_t SPVM__Sys__IO__fread(SPVM_ENV* env, SPVM_VALUE* stack) {
   
   FILE* stream = env->get_pointer(env, stack, obj_stream);
   
-  int32_t fread_length = fread(buffer, size, data_length, stream);
+  int32_t fd = fileno(stream);
+
+  if (fd == -1) {
+    env->die(env, stack, "[System Error]fileno failed:%s.", env->strerror(env, stack, errno, 0), FILE_NAME, __LINE__);
+    return SPVM_NATIVE_C_CLASS_ID_ERROR_SYSTEM;
+  }
   
-  stack[0].ival = fread_length;
+  stack[0].ival = fd;
   
   return 0;
 }
 
-int32_t SPVM__Sys__IO__fwrite(SPVM_ENV* env, SPVM_VALUE* stack) {
+int32_t SPVM__Sys__IO__fread(SPVM_ENV* env, SPVM_VALUE* stack) {
 
   int32_t e = 0;
   
-  void* obj_buffer = stack[0].oval;
+  void* obj_ptr = stack[0].oval;
   
-  if (!obj_buffer) {
-    return env->die(env, stack, "The buffer must be defined", FILE_NAME, __LINE__);
+  if (!obj_ptr) {
+    return env->die(env, stack, "The read buffer(ptr) must be defined", FILE_NAME, __LINE__);
   }
   
-  char* buffer = (char*)env->get_chars(env, stack, obj_buffer);
-  int32_t buffer_length = env->length(env, stack, obj_buffer);
+  char* ptr = (char*)env->get_chars(env, stack, obj_ptr);
+  int32_t ptr_length = env->length(env, stack, obj_ptr);
   
   int32_t size = stack[1].ival;
   if (!(size >= 0)) {
     return env->die(env, stack, "The size must be more than or equal to 0", FILE_NAME, __LINE__);
   }
   
-  int32_t data_length = stack[2].ival;
-  if (!(data_length >= 0)) {
-    return env->die(env, stack, "The data length must be more than or equal to 0", FILE_NAME, __LINE__);
+  int32_t nmemb = stack[2].ival;
+  if (!(nmemb >= 0)) {
+    return env->die(env, stack, "The data length(nmemb) must be more than or equal to 0", FILE_NAME, __LINE__);
   }
   
-  if (size == 0 || data_length == 0) {
+  if (size == 0 || nmemb == 0) {
     stack[0].ival = 0;
     return 0;
   }
   
-  if (!((buffer_length / size) <= data_length)) {
-    return env->die(env, stack, "The buffer length / the size must be less than or equal to the data length", FILE_NAME, __LINE__);
+  if (!((ptr_length / size) <= nmemb)) {
+    return env->die(env, stack, "The read buffer length / the size must be less than or equal to the data length", FILE_NAME, __LINE__);
   }
   
   void* obj_stream = stack[3].oval;
@@ -329,9 +291,58 @@ int32_t SPVM__Sys__IO__fwrite(SPVM_ENV* env, SPVM_VALUE* stack) {
   
   FILE* stream = env->get_pointer(env, stack, obj_stream);
   
-  int32_t fwrite_length = fwrite(buffer, size, data_length, stream);
+  int32_t read_length = fread(ptr, size, nmemb, stream);
   
-  stack[0].ival = fwrite_length;
+  stack[0].ival = read_length;
+  
+  return 0;
+}
+
+int32_t SPVM__Sys__IO__feof(SPVM_ENV* env, SPVM_VALUE* stack) {
+  
+  void* obj_stream = stack[0].oval;
+  
+  if (!obj_stream) {
+    return env->die(env, stack, "The file stream must be defined", FILE_NAME, __LINE__);
+  }
+  
+  FILE* stream = env->get_pointer(env, stack, obj_stream);
+  
+  int32_t ret = feof(stream);
+  
+  stack[0].ival = ret;
+  
+  return 0;
+}
+
+int32_t SPVM__Sys__IO__ferror(SPVM_ENV* env, SPVM_VALUE* stack) {
+  
+  void* obj_stream = stack[0].oval;
+  
+  if (!obj_stream) {
+    return env->die(env, stack, "The file stream must be defined", FILE_NAME, __LINE__);
+  }
+  
+  FILE* stream = env->get_pointer(env, stack, obj_stream);
+  
+  int32_t ret = ferror(stream);
+  
+  stack[0].ival = ret;
+  
+  return 0;
+}
+
+int32_t SPVM__Sys__IO__clearerr(SPVM_ENV* env, SPVM_VALUE* stack) {
+  
+  void* obj_stream = stack[0].oval;
+  
+  if (!obj_stream) {
+    return env->die(env, stack, "The file stream must be defined", FILE_NAME, __LINE__);
+  }
+  
+  FILE* stream = env->get_pointer(env, stack, obj_stream);
+  
+  clearerr(stream);
   
   return 0;
 }
@@ -359,14 +370,14 @@ int32_t SPVM__Sys__IO__fgets(SPVM_ENV* env, SPVM_VALUE* stack) {
 
   int32_t e = 0;
   
-  void* obj_buffer = stack[0].oval;
+  void* obj_s = stack[0].oval;
   
-  if (!obj_buffer) {
-    return env->die(env, stack, "The buffer must be defined", FILE_NAME, __LINE__);
+  if (!obj_s) {
+    return env->die(env, stack, "The read buffer(s) must be defined", FILE_NAME, __LINE__);
   }
   
-  char* buffer = (char*)env->get_chars(env, stack, obj_buffer);
-  int32_t buffer_length = env->length(env, stack, obj_buffer);
+  char* s = (char*)env->get_chars(env, stack, obj_s);
+  int32_t s_length = env->length(env, stack, obj_s);
   
   int32_t size = stack[1].ival;
   if (!(size >= 0)) {
@@ -381,10 +392,10 @@ int32_t SPVM__Sys__IO__fgets(SPVM_ENV* env, SPVM_VALUE* stack) {
   
   FILE* stream = env->get_pointer(env, stack, obj_stream);
   
-  char* ret_buffer = fgets(buffer, size, stream);
+  char* ret_s = fgets(s, size, stream);
   
-  if (ret_buffer) {
-    stack[0].oval = obj_buffer;
+  if (ret_s) {
+    stack[0].oval = obj_s;
   }
   else {
     stack[0].oval = NULL;
@@ -393,6 +404,53 @@ int32_t SPVM__Sys__IO__fgets(SPVM_ENV* env, SPVM_VALUE* stack) {
   return 0;
 }
 
+int32_t SPVM__Sys__IO__fwrite(SPVM_ENV* env, SPVM_VALUE* stack) {
+
+  int32_t e = 0;
+  
+  void* obj_ptr = stack[0].oval;
+  
+  if (!obj_ptr) {
+    return env->die(env, stack, "The write-buffer(ptr) must be defined", FILE_NAME, __LINE__);
+  }
+  
+  char* ptr = (char*)env->get_chars(env, stack, obj_ptr);
+  int32_t ptr_length = env->length(env, stack, obj_ptr);
+  
+  int32_t size = stack[1].ival;
+  if (!(size >= 0)) {
+    return env->die(env, stack, "The size must be more than or equal to 0", FILE_NAME, __LINE__);
+  }
+  
+  int32_t nmemb = stack[2].ival;
+  if (!(nmemb >= 0)) {
+    return env->die(env, stack, "The data length(nmemb) must be more than or equal to 0", FILE_NAME, __LINE__);
+  }
+  
+  if (size == 0 || nmemb == 0) {
+    stack[0].ival = 0;
+    return 0;
+  }
+  
+  if (!((ptr_length / size) <= nmemb)) {
+    return env->die(env, stack, "The write-buffer length / the size must be less than or equal to the data length", FILE_NAME, __LINE__);
+  }
+  
+  void* obj_stream = stack[3].oval;
+  
+  if (!obj_stream) {
+    return env->die(env, stack, "The file stream must be defined", FILE_NAME, __LINE__);
+  }
+  
+  FILE* stream = env->get_pointer(env, stack, obj_stream);
+  
+  int32_t fwrite_length = fwrite(ptr, size, nmemb, stream);
+  
+  stack[0].ival = fwrite_length;
+  
+  return 0;
+}
+
 int32_t SPVM__Sys__IO__fclose(SPVM_ENV* env, SPVM_VALUE* stack) {
   
   void* obj_stream = stack[0].oval;
@@ -510,78 +568,284 @@ int32_t SPVM__Sys__IO__flock(SPVM_ENV* env, SPVM_VALUE* stack) {
 #endif
 }
 
-int32_t SPVM__Sys__IO__read(SPVM_ENV* env, SPVM_VALUE* stack) {
+int32_t SPVM__Sys__IO__mkdir(SPVM_ENV* env, SPVM_VALUE* stack) {
   
-  int32_t e = 0;
+  void* obj_path = stack[0].oval;
   
-  int32_t fd = stack[0].ival;
-  
-  void* obj_buffer = stack[1].oval;
-  
-  if (!obj_buffer) {
-    return env->die(env, stack, "The buffer must be defined", FILE_NAME, __LINE__);
+  if (!obj_path) {
+    return env->die(env, stack, "The path must be defined", FILE_NAME, __LINE__);
   }
   
-  char* buffer = (char*)env->get_chars(env, stack, obj_buffer);
-  int32_t buffer_length = env->length(env, stack, obj_buffer);
+  const char* path = env->get_chars(env, stack, obj_path);
 
-  int32_t count = stack[2].ival;
-  
-  if (!(count >= 0)) {
-    return env->die(env, stack, "The count must be more than or equal to 0", FILE_NAME, __LINE__);
-  }
-  
-  if (!(count < buffer_length)) {
-    return env->die(env, stack, "The count must be less than the length of the buffer", FILE_NAME, __LINE__);
-  }
-  
-  int32_t read_length = read(fd, buffer, count);
-  if (read_length == -1) {
-    env->die(env, stack, "[System Error]read failed:%s.", env->strerror(env, stack, errno, 0), FILE_NAME, __LINE__);
+  int32_t mode = stack[1].ival;
+
+#ifdef _WIN32
+  int32_t status = mkdir(path);
+#else
+  int32_t status = mkdir(path, mode);
+#endif
+
+  if (status == -1) {
+    env->die(env, stack, "[System Error]mkdir failed:%s.", env->strerror(env, stack, errno, 0), FILE_NAME, __LINE__);
     return SPVM_NATIVE_C_CLASS_ID_ERROR_SYSTEM;
   }
   
-  stack[0].ival = read_length;
+  stack[0].ival = status;
   
   return 0;
 }
 
-int32_t SPVM__Sys__IO__write(SPVM_ENV* env, SPVM_VALUE* stack) {
+int32_t SPVM__Sys__IO__umask(SPVM_ENV* env, SPVM_VALUE* stack) {
   
-  int32_t e = 0;
+  int32_t mode = stack[0].ival;
   
-  int32_t fd = stack[0].ival;
+  int32_t old_mode = umask(mode);
   
-  void* obj_buffer = stack[1].oval;
+  stack[0].ival = old_mode;
   
-  if (!obj_buffer) {
-    return env->die(env, stack, "The buffer must be defined", FILE_NAME, __LINE__);
-  }
-  
-  char* buffer = (char*)env->get_chars(env, stack, obj_buffer);
-  int32_t buffer_length = env->length(env, stack, obj_buffer);
+  return 0;
+}
 
-  int32_t count = stack[2].ival;
+int32_t SPVM__Sys__IO__rmdir(SPVM_ENV* env, SPVM_VALUE* stack) {
   
-  if (!(count >= 0)) {
-    return env->die(env, stack, "The count must be more than or equal to 0", FILE_NAME, __LINE__);
+  void* obj_path = stack[0].oval;
+  
+  if (!obj_path) {
+    return env->die(env, stack, "The path must be defined", FILE_NAME, __LINE__);
   }
   
-  if (!(count < buffer_length)) {
-    return env->die(env, stack, "The count must be less than the length of the buffer", FILE_NAME, __LINE__);
-  }
-  
-  int32_t write_length = write(fd, buffer, count);
-  if (write_length == -1) {
-    env->die(env, stack, "[System Error]write failed:%s.", env->strerror(env, stack, errno, 0), FILE_NAME, __LINE__);
+  const char* path = env->get_chars(env, stack, obj_path);
+  int32_t status = rmdir(path);
+  if (status == -1) {
+    env->die(env, stack, "[System Error]rmdir failed:%s.", env->strerror(env, stack, errno, 0), FILE_NAME, __LINE__);
     return SPVM_NATIVE_C_CLASS_ID_ERROR_SYSTEM;
   }
   
-  stack[0].ival = write_length;
+  stack[0].ival = status;
   
   return 0;
 }
 
+int32_t SPVM__Sys__IO__unlink(SPVM_ENV* env, SPVM_VALUE* stack) {
+  
+  void* obj_pathname = stack[0].oval;
+
+  if (!obj_pathname) {
+    return env->die(env, stack, "The pathname must be defined", FILE_NAME, __LINE__);
+  }
+  
+  const char* pathname = env->get_chars(env, stack, obj_pathname);
+  int32_t status = unlink(pathname);
+  if (status == -1) {
+    env->die(env, stack, "[System Error]unlink failed:%s.", env->strerror(env, stack, errno, 0), FILE_NAME, __LINE__);
+    return SPVM_NATIVE_C_CLASS_ID_ERROR_SYSTEM;
+  }
+
+  stack[0].ival = status;
+  
+  return 0;
+}
+
+int32_t SPVM__Sys__IO__rename(SPVM_ENV* env, SPVM_VALUE* stack) {
+  
+  int32_t e = 0;
+  
+  void* obj_oldpath = stack[0].oval;
+  
+  if (!obj_oldpath) {
+    return env->die(env, stack, "The old path must be defined", FILE_NAME, __LINE__);
+  }
+  
+  const char* oldpath = env->get_chars(env, stack, obj_oldpath);
+
+  void* obj_newpath = stack[1].oval;
+  
+  if (!obj_newpath) {
+    return env->die(env, stack, "The new path must be defined", FILE_NAME, __LINE__);
+  }
+  
+  const char* newpath = env->get_chars(env, stack, obj_newpath);
+  
+  int32_t status = rename(oldpath, newpath);
+  if (status == -1) {
+    env->die(env, stack, "[System Error]rename failed:%s.", env->strerror(env, stack, errno, 0), FILE_NAME, __LINE__);
+    return SPVM_NATIVE_C_CLASS_ID_ERROR_SYSTEM;
+  }
+  
+  stack[0].ival = status;
+  
+  return 0;
+}
+
+int32_t SPVM__Sys__IO__getcwd(SPVM_ENV* env, SPVM_VALUE* stack) {
+  
+  int32_t e = 0;
+  
+  void* obj_buf = stack[0].oval;
+
+  int32_t size = stack[1].ival;
+
+  if (!(size >= 0)) {
+    return env->die(env, stack, "The size must be greater than or equal to 0", FILE_NAME, __LINE__);
+  }
+  
+  char* ret_buf;
+  if (obj_buf) {
+    char* buf = (char*)env->get_chars(env, stack, obj_buf);
+    int32_t buf_length = env->length(env, stack, obj_buf);
+    if (!(size <= buf_length)) {
+      return env->die(env, stack, "The size must be less than or equal to the lenght of the buffer", FILE_NAME, __LINE__);
+    }
+    
+    ret_buf = getcwd(buf, size);
+  }
+  else {
+    ret_buf = getcwd(NULL, size);
+    if (ret_buf) {
+      obj_buf = env->new_string(env, stack, ret_buf, strlen(ret_buf));
+      free(ret_buf);
+    }
+  }
+  
+  if (!ret_buf) {
+    env->die(env, stack, "[System Error]getcwd failed:%s.", env->strerror(env, stack, errno, 0), FILE_NAME, __LINE__);
+    return SPVM_NATIVE_C_CLASS_ID_ERROR_SYSTEM;
+  }
+  
+  stack[0].oval = obj_buf;
+  
+  return 0;
+}
+
+int32_t SPVM__Sys__IO___getdcwd(SPVM_ENV* env, SPVM_VALUE* stack) {
+#ifdef _WIN32
+  int32_t e = 0;
+  
+  int32_t drive = stack[0].ival;
+  
+  void* obj_buffer = stack[1].oval;
+
+  int32_t maxlen = stack[2].ival;
+
+  if (!(maxlen > 0)) {
+    return env->die(env, stack, "The maxlen must be greater than 0", FILE_NAME, __LINE__);
+  }
+  
+  char* ret_buffer;
+  if (obj_buffer) {
+    char* buffer = (char*)env->get_chars(env, stack, obj_buffer);
+    int32_t buffer_length = env->length(env, stack, obj_buffer);
+    if (!(maxlen <= buffer_length)) {
+      return env->die(env, stack, "The maxlen must be less than or equal to the lenght of the bufferfer", FILE_NAME, __LINE__);
+    }
+    
+    ret_buffer = _getdcwd(drive, buffer, maxlen);
+  }
+  else {
+    ret_buffer = _getdcwd(drive, NULL, maxlen);
+    if (ret_buffer) {
+      obj_buffer = env->new_string(env, stack, ret_buffer, strlen(ret_buffer));
+      free(ret_buffer);
+    }
+  }
+  
+  if (!ret_buffer) {
+    env->die(env, stack, "[System Error]_getdcwd failed:%s.", env->strerror(env, stack, errno, 0), FILE_NAME, __LINE__);
+    return SPVM_NATIVE_C_CLASS_ID_ERROR_SYSTEM;
+  }
+  
+  stack[0].oval = obj_buffer;
+  
+  return 0;
+#else
+  env->die(env, stack, "_getdcwd is not supported on this system", FILE_NAME, __LINE__);
+  return SPVM_NATIVE_C_CLASS_ID_ERROR_NOT_SUPPORTED;
+#endif
+}
+
+int32_t SPVM__Sys__IO__realpath(SPVM_ENV* env, SPVM_VALUE* stack) {
+#ifdef _WIN32
+  env->die(env, stack, "realpath is not supported on this system", FILE_NAME, __LINE__);
+  return SPVM_NATIVE_C_CLASS_ID_ERROR_NOT_SUPPORTED;
+#else
+  
+  void* obj_path = stack[0].oval;
+  
+  if (!obj_path) {
+    return env->die(env, stack, "The path must be defined", FILE_NAME, __LINE__);
+  }
+
+  const char* path = env->get_chars(env, stack, obj_path);
+
+  void* obj_resolved_path = stack[1].oval;
+  
+  char* ret_resolved_path;
+  if (obj_resolved_path) {
+    char* resolved_path = (char*)env->get_chars(env, stack, obj_resolved_path);
+    ret_resolved_path = realpath(path, resolved_path);
+  }
+  else {
+    ret_resolved_path = realpath(path, NULL);
+    if (ret_resolved_path) {
+      obj_resolved_path = env->new_string(env, stack, ret_resolved_path, strlen(ret_resolved_path));
+      free(ret_resolved_path);
+    }
+  }
+  
+  if (!ret_resolved_path) {
+    env->die(env, stack, "[System Error]realpath failed:%s.", env->strerror(env, stack, errno, 0), FILE_NAME, __LINE__);
+    return SPVM_NATIVE_C_CLASS_ID_ERROR_SYSTEM;
+  }
+  
+  stack[0].oval = obj_resolved_path;
+
+  return 0;
+#endif
+}
+
+int32_t SPVM__Sys__IO___fullpath(SPVM_ENV* env, SPVM_VALUE* stack) {
+#ifdef _WIN32
+  
+  void* obj_absPath = stack[0].oval;
+  
+  void* obj_relPath = stack[1].oval;
+
+  if (!obj_relPath) {
+    return env->die(env, stack, "The relPath must be defined", FILE_NAME, __LINE__);
+  }
+  
+  char* relPath = (char*)env->get_chars(env, stack, obj_relPath);
+  
+  int32_t maxLength = stack[2].ival;
+  
+  char* ret_absPath;
+  if (obj_absPath) {
+    char* absPath = (char*)env->get_chars(env, stack, obj_absPath);
+    ret_absPath = _fullpath(absPath, relPath, maxLength);
+  }
+  else {
+    ret_absPath = _fullpath(NULL, relPath, 0);
+    if (ret_absPath) {
+      obj_absPath = env->new_string(env, stack, ret_absPath, strlen(ret_absPath));
+      free(ret_absPath);
+    }
+  }
+  
+  if (!ret_absPath) {
+    env->die(env, stack, "[System Error]_fullpath failed:%s.", env->strerror(env, stack, errno, 0), FILE_NAME, __LINE__);
+    return SPVM_NATIVE_C_CLASS_ID_ERROR_SYSTEM;
+  }
+  
+  stack[0].oval = obj_absPath;
+
+  return 0;
+#else
+  env->die(env, stack, "_fullpath is not supported on this system", FILE_NAME, __LINE__);
+  return SPVM_NATIVE_C_CLASS_ID_ERROR_NOT_SUPPORTED;
+#endif
+}
+
 int32_t SPVM__Sys__IO__chdir(SPVM_ENV* env, SPVM_VALUE* stack) {
   
   void* obj_path = stack[0].oval;
@@ -656,7 +920,7 @@ int32_t SPVM__Sys__IO__chown(SPVM_ENV* env, SPVM_VALUE* stack) {
 #endif
 }
 
-int32_t SPVM__Sys__IO__mkdir(SPVM_ENV* env, SPVM_VALUE* stack) {
+int32_t SPVM__Sys__IO__truncate(SPVM_ENV* env, SPVM_VALUE* stack) {
   
   void* obj_path = stack[0].oval;
   
@@ -666,16 +930,15 @@ int32_t SPVM__Sys__IO__mkdir(SPVM_ENV* env, SPVM_VALUE* stack) {
   
   const char* path = env->get_chars(env, stack, obj_path);
 
-  int32_t mode = stack[1].ival;
-
-#ifdef _WIN32
-  int32_t status = mkdir(path);
-#else
-  int32_t status = mkdir(path, mode);
-#endif
+  int64_t length = stack[1].lval;
+  
+  if (!(length >= 0)) {
+    return env->die(env, stack, "The length must be less than or equal to 0", FILE_NAME, __LINE__);
+  }
 
+  int32_t status = truncate(path, length);
   if (status == -1) {
-    env->die(env, stack, "[System Error]mkdir failed:%s.", env->strerror(env, stack, errno, 0), FILE_NAME, __LINE__);
+    env->die(env, stack, "[System Error]truncate failed:%s.", env->strerror(env, stack, errno, 0), FILE_NAME, __LINE__);
     return SPVM_NATIVE_C_CLASS_ID_ERROR_SYSTEM;
   }
   
@@ -684,6 +947,87 @@ int32_t SPVM__Sys__IO__mkdir(SPVM_ENV* env, SPVM_VALUE* stack) {
   return 0;
 }
 
+int32_t SPVM__Sys__IO__readlink(SPVM_ENV* env, SPVM_VALUE* stack) {
+#ifdef _WIN32
+  env->die(env, stack, "readlink is not supported on this system", FILE_NAME, __LINE__);
+  return SPVM_NATIVE_C_CLASS_ID_ERROR_NOT_SUPPORTED;
+#else
+  int32_t e = 0;
+  
+  void* obj_path = stack[0].oval;
+  
+  if (!obj_path) {
+    return env->die(env, stack, "The old path must be defined", FILE_NAME, __LINE__);
+  }
+  
+  const char* path = env->get_chars(env, stack, obj_path);
+
+  void* obj_buffer = stack[1].oval;
+  
+  if (!obj_buffer) {
+    return env->die(env, stack, "The new path must be defined", FILE_NAME, __LINE__);
+  }
+  
+  char* buffer = (char*)env->get_chars(env, stack, obj_buffer);
+  int32_t buffer_length = env->length(env, stack, obj_buffer);
+  
+  int32_t arg_buffer_length = stack[2].ival;
+  
+  if (!(arg_buffer_length >= 0)) {
+    return env->die(env, stack, "The buffer length of the argument must be greater than or equal to 0", FILE_NAME, __LINE__);
+  }
+  
+  if (!(arg_buffer_length <= buffer_length)) {
+    return env->die(env, stack, "The buffer length of the argument must be less than or equal to the length of the buffer", FILE_NAME, __LINE__);
+  }
+  
+  int32_t placed_length = readlink(path, buffer, arg_buffer_length);
+  if (placed_length == -1) {
+    env->die(env, stack, "[System Error]readlink failed:%s.", env->strerror(env, stack, errno, 0), FILE_NAME, __LINE__);
+    return SPVM_NATIVE_C_CLASS_ID_ERROR_SYSTEM;
+  }
+  
+  stack[0].ival = placed_length;
+  
+  return 0;
+#endif
+}
+
+int32_t SPVM__Sys__IO__symlink(SPVM_ENV* env, SPVM_VALUE* stack) {
+#ifdef _WIN32
+  env->die(env, stack, "symlink is not supported on this system", FILE_NAME, __LINE__);
+  return SPVM_NATIVE_C_CLASS_ID_ERROR_NOT_SUPPORTED;
+#else
+  int32_t e = 0;
+  
+  void* obj_oldpath = stack[0].oval;
+  
+  if (!obj_oldpath) {
+    return env->die(env, stack, "The oldpath must be defined", FILE_NAME, __LINE__);
+  }
+  
+  const char* oldpath = env->get_chars(env, stack, obj_oldpath);
+
+  void* obj_newpath = stack[1].oval;
+  
+  if (!obj_newpath) {
+    return env->die(env, stack, "The link path must be defined", FILE_NAME, __LINE__);
+  }
+  
+  const char* newpath = env->get_chars(env, stack, obj_newpath);
+  
+  int32_t status = symlink(oldpath, newpath);
+  if (status == -1) {
+    env->die(env, stack, "[System Error]symlink failed:%s.", env->strerror(env, stack, errno, 0), FILE_NAME, __LINE__);
+    return SPVM_NATIVE_C_CLASS_ID_ERROR_SYSTEM;
+  }
+  
+  stack[0].ival = status;
+  
+  return 0;
+#endif
+}
+
 static const int DIR_STREAM_CLOSED_INDEX = 0;
 
 int32_t SPVM__Sys__IO__opendir(SPVM_ENV* env, SPVM_VALUE* stack) {
@@ -824,197 +1168,28 @@ int32_t SPVM__Sys__IO__seekdir(SPVM_ENV* env, SPVM_VALUE* stack) {
   return 0;
 }
 
-int32_t SPVM__Sys__IO__truncate(SPVM_ENV* env, SPVM_VALUE* stack) {
-  
-  void* obj_path = stack[0].oval;
-  
-  if (!obj_path) {
-    return env->die(env, stack, "The path must be defined", FILE_NAME, __LINE__);
-  }
-  
-  const char* path = env->get_chars(env, stack, obj_path);
-
-  int64_t offset = stack[1].ival;
-  
-  if (!(offset >= 0)) {
-    return env->die(env, stack, "The offset must be less than or equal to 0", FILE_NAME, __LINE__);
-  }
-
-  int32_t status = truncate(path, offset);
-  if (status == -1) {
-    env->die(env, stack, "[System Error]truncate failed:%s.", env->strerror(env, stack, errno, 0), FILE_NAME, __LINE__);
-    return SPVM_NATIVE_C_CLASS_ID_ERROR_SYSTEM;
-  }
-  
-  return 0;
-}
-
-int32_t SPVM__Sys__IO__lseek(SPVM_ENV* env, SPVM_VALUE* stack) {
-  
-  int32_t fd = stack[0].ival;
-  
-  int64_t offset = stack[1].lval;
-  
-  if (!(offset >= 0)) {
-    return env->die(env, stack, "The offset must be greater than or equal to 0", FILE_NAME, __LINE__);
-  }
-
-  int32_t whence = stack[2].ival;
-
-  int64_t cur_offset = lseek(fd, offset, whence);
-  if (cur_offset == -1) {
-    env->die(env, stack, "[System Error]lseek failed:%s.", env->strerror(env, stack, errno, 0), FILE_NAME, __LINE__);
-    return SPVM_NATIVE_C_CLASS_ID_ERROR_SYSTEM;
-  }
-  
-  return 0;
-}
-
-int32_t SPVM__Sys__IO__umask(SPVM_ENV* env, SPVM_VALUE* stack) {
-  
-  int32_t mode = stack[0].ival;
-  
-  int32_t old_mode = umask(mode);
-  
-  stack[0].ival = old_mode;
-  
-  return 0;
-}
-
-int32_t SPVM__Sys__IO__rename(SPVM_ENV* env, SPVM_VALUE* stack) {
-  
-  int32_t e = 0;
-  
-  void* obj_path = stack[0].oval;
-  
-  if (!obj_path) {
-    return env->die(env, stack, "The old path must be defined", FILE_NAME, __LINE__);
-  }
-  
-  const char* path = env->get_chars(env, stack, obj_path);
-
-  void* obj_new_path = stack[1].oval;
-  
-  if (!obj_new_path) {
-    return env->die(env, stack, "The new path must be defined", FILE_NAME, __LINE__);
-  }
-  
-  const char* new_path = env->get_chars(env, stack, obj_new_path);
-  
-  int32_t status = rename(path, new_path);
-  if (status == -1) {
-    env->die(env, stack, "[System Error]rename failed:%s.", env->strerror(env, stack, errno, 0), FILE_NAME, __LINE__);
-    return SPVM_NATIVE_C_CLASS_ID_ERROR_SYSTEM;
-  }
-  
-  stack[0].ival = status;
-  
-  return 0;
-}
-
-int32_t SPVM__Sys__IO__readlink(SPVM_ENV* env, SPVM_VALUE* stack) {
-#ifdef _WIN32
-  env->die(env, stack, "readlink is not supported on this system", FILE_NAME, __LINE__);
-  return SPVM_NATIVE_C_CLASS_ID_ERROR_NOT_SUPPORTED;
-#else
-  int32_t e = 0;
-  
-  void* obj_path = stack[0].oval;
-  
-  if (!obj_path) {
-    return env->die(env, stack, "The old path must be defined", FILE_NAME, __LINE__);
-  }
-  
-  const char* path = env->get_chars(env, stack, obj_path);
-
-  void* obj_buffer = stack[1].oval;
-  
-  if (!obj_buffer) {
-    return env->die(env, stack, "The new path must be defined", FILE_NAME, __LINE__);
-  }
-  
-  char* buffer = (char*)env->get_chars(env, stack, obj_buffer);
-  int32_t buffer_length = env->length(env, stack, obj_buffer);
-  
-  int32_t arg_buffer_length = stack[2].ival;
-  
-  if (!(arg_buffer_length >= 0)) {
-    return env->die(env, stack, "The buffer length of the argument must be greater than or equal to 0", FILE_NAME, __LINE__);
-  }
-  
-  if (!(arg_buffer_length <= buffer_length)) {
-    return env->die(env, stack, "The buffer length of the argument must be less than or equal to the length of the buffer", FILE_NAME, __LINE__);
-  }
-  
-  int32_t placed_length = readlink(path, buffer, arg_buffer_length);
-  if (placed_length == -1) {
-    env->die(env, stack, "[System Error]readlink failed:%s.", env->strerror(env, stack, errno, 0), FILE_NAME, __LINE__);
-    return SPVM_NATIVE_C_CLASS_ID_ERROR_SYSTEM;
-  }
-  
-  stack[0].ival = placed_length;
-  
-  return 0;
-#endif
-}
-
-int32_t SPVM__Sys__IO__symlink(SPVM_ENV* env, SPVM_VALUE* stack) {
-#ifdef _WIN32
-  env->die(env, stack, "symlink is not supported on this system", FILE_NAME, __LINE__);
-  return SPVM_NATIVE_C_CLASS_ID_ERROR_NOT_SUPPORTED;
-#else
-  int32_t e = 0;
-  
-  void* obj_target = stack[0].oval;
-  
-  if (!obj_target) {
-    return env->die(env, stack, "The target must be defined", FILE_NAME, __LINE__);
-  }
-  
-  const char* target = env->get_chars(env, stack, obj_target);
-
-  void* obj_link_path = stack[1].oval;
-  
-  if (!obj_link_path) {
-    return env->die(env, stack, "The link path must be defined", FILE_NAME, __LINE__);
-  }
-  
-  const char* link_path = env->get_chars(env, stack, obj_link_path);
-  
-  int32_t status = symlink(target, link_path);
-  if (status == -1) {
-    env->die(env, stack, "[System Error]symlink failed:%s.", env->strerror(env, stack, errno, 0), FILE_NAME, __LINE__);
-    return SPVM_NATIVE_C_CLASS_ID_ERROR_SYSTEM;
-  }
-  
-  stack[0].ival = status;
-  
-  return 0;
-#endif
-}
-
 int32_t SPVM__Sys__IO__utime(SPVM_ENV* env, SPVM_VALUE* stack) {
   
   int32_t e = 0;
   
-  void* obj_file = stack[0].oval;
+  void* obj_filename = stack[0].oval;
   
-  if (!obj_file) {
-    return env->die(env, stack, "The file must be defined", FILE_NAME, __LINE__);
+  if (!obj_filename) {
+    return env->die(env, stack, "The filename must be defined", FILE_NAME, __LINE__);
   }
   
-  const char* file = env->get_chars(env, stack, obj_file);
+  const char* filename = env->get_chars(env, stack, obj_filename);
 
-  void* obj_buffer = stack[1].oval;
-  struct utimbuf* st_buffer;
-  if (obj_buffer) {
-    st_buffer = env->get_pointer(env, stack, obj_buffer);
+  void* obj_times = stack[1].oval;
+  struct utimbuf* st_times;
+  if (obj_times) {
+    st_times = env->get_pointer(env, stack, obj_times);
   }
   else {
-    st_buffer = NULL;
+    st_times = NULL;
   }
   
-  int32_t status = utime(file, st_buffer);
+  int32_t status = utime(filename, st_times);
   if (status == -1) {
     env->die(env, stack, "[System Error]utime failed:%s.", env->strerror(env, stack, errno, 0), FILE_NAME, __LINE__);
     return SPVM_NATIVE_C_CLASS_ID_ERROR_SYSTEM;
