@@ -16,22 +16,6 @@ C<Sys::IO> is the class for the file IO.
 
 =head1 Class Methods
 
-=head2 rmdir
-
-  static method rmdir : int ($path : string);
-
-rmdir() deletes a directory, which must be empty.
-
-See the detail of the L<rmdir|https://linux.die.net/man/2/rmdir> function in the case of Linux.
-
-=head2 unlink
-
-  static method unlink : int ($path : string);
-
-unlink() deletes a name from the file system. If that name was the last link to a file and no processes have the file open the file is deleted and the space it was using is made available for reuse.
-
-See the detail of the L<unlink|https://linux.die.net/man/2/unlink> function in the case of Linux.
-
 =head2 open
 
   static method open : int ($path : string, $flags : int, $mode = 0 : int);
@@ -42,6 +26,32 @@ See the detail of the L<open|https://linux.die.net/man/2/open> function in the c
 
 See L<Sys::IO::Constant|SPVM::Sys::IO::Constant> about the constant value for the flags and the mode.
 
+=head2 read
+
+  static method read : int ($fd : int, $buf : mutable string, $count : int);
+
+read() attempts to read up to count bytes from file descriptor fd into the buffer starting at buf.
+
+See the detail of the L<read|https://linux.die.net/man/2/read> function in the case of Linux.
+
+=head2 write
+
+  static method write : int ($fd : int, $buffer : string, $count : int);
+
+write() writes up to count bytes from the buffer pointed buf to the file referred to by the file descriptor fd.
+
+See the detail of the L<write|https://linux.die.net/man/2/write> function in the case of Linux.
+
+=head2 lseek
+
+  static method lseek : long ($fd : int, $offset : long, $whence : int);
+
+The lseek() function repositions the offset of the open file associated with the file descriptor fd to the argument offset according to the directive whence as follows:
+
+See the detail of the L<lseek|https://linux.die.net/man/2/lseek> function in the case of Linux.
+
+See L<Sys::IO::Constant|SPVM::Sys::IO::Constant> about the constant value for the whence.
+
 =head2 close
 
   static method close : int ($fd : int);
@@ -50,46 +60,6 @@ close() closes a file descriptor, so that it no longer refers to any file and ma
 
 See the detail of the L<close|https://linux.die.net/man/2/close> function in the case of Linux.
 
-=head2 clearerr
-
-  static method clearerr : void ($stream : Sys::IO::FileStream);
-
-The function clearerr() clears the end-of-file and error indicators for the stream pointed to by stream.
-
-See the detail of the L<clearerr|https://linux.die.net/man/3/clearerr> function in the case of Linux.
-
-The file stream is a L<Sys::IO::FileStream|SPVM::Sys::IO::FileStream> object.
-
-=head2 feof
-
-  static method feof : int ($stream : Sys::IO::FileStream);
-
-The function feof() tests the end-of-file indicator for the stream pointed to by stream, returning nonzero if it is set. The end-of-file indicator can only be cleared by the function clearerr().
-
-See the detail of the L<feof|https://linux.die.net/man/3/feof> function in the case of Linux.
-
-The file stream is a L<Sys::IO::FileStream|SPVM::Sys::IO::FileStream> object.
-
-=head2 ferror
-
-  static method ferror : int ($stream : Sys::IO::FileStream);
-
-The function ferror() tests the error indicator for the stream pointed to by stream, returning nonzero if it is set. The error indicator can only be reset by the clearerr() function.
-
-See the detail of the L<ferror|https://linux.die.net/man/3/ferror> function in the case of Linux.
-
-The file stream is a L<Sys::IO::FileStream|SPVM::Sys::IO::FileStream> object.
-
-=head2 fileno
-
-  static method fileno : int ($stream : Sys::IO::FileStream);
-
-The function fileno() examines the argument stream and returns its integer descriptor.
-
-See the detail of the L<fileno|https://linux.die.net/man/3/fileno> function in the case of Linux.
-
-The file stream is a L<Sys::IO::FileStream|SPVM::Sys::IO::FileStream> object.
-
 =head2 fopen
 
   static method fopen : Sys::IO::FileStream ($path : string, $mode : string);
@@ -110,21 +80,51 @@ See the detail of the L<fdopen|https://linux.die.net/man/3/fdopen> function in t
 
 The return value is a L<Sys::IO::FileStream|SPVM::Sys::IO::FileStream> object.
 
+=head2 fileno
+
+  static method fileno : int ($stream : Sys::IO::FileStream);
+
+The function fileno() examines the argument stream and returns its integer descriptor.
+
+See the detail of the L<fileno|https://linux.die.net/man/3/fileno> function in the case of Linux.
+
+The file stream is a L<Sys::IO::FileStream|SPVM::Sys::IO::FileStream> object.
+
 =head2 fread
 
-  static method fread : int ($buffer : mutable string, $size : int, $data_length : int, $stream : Sys::IO::FileStream);
+  static method fread : int ($ptr : mutable string, $size : int, $nmemb : int, $stream : Sys::IO::FileStream);
 
 The function fread() reads nmemb elements of data, each size bytes long, from the stream pointed to by stream, storing them at the location given by ptr.
 
 See the detail of the L<fread|https://linux.die.net/man/3/fread> function in the case of Linux.
 
-=head2 fwrite
+=head2 feof
 
-  static method fwrite : int ($buffer : mutable string, $size : int, $data_length : int, $stream : Sys::IO::FileStream);
+  static method feof : int ($stream : Sys::IO::FileStream);
 
-The function fwrite() writes nmemb elements of data, each size bytes long, to the stream pointed to by stream, obtaining them from the location given by ptr.
+The function feof() tests the end-of-file indicator for the stream pointed to by stream, returning nonzero if it is set. The end-of-file indicator can only be cleared by the function clearerr().
 
-See the detail of the L<fread|https://linux.die.net/man/3/fwrite> function in the case of Linux.
+See the detail of the L<feof|https://linux.die.net/man/3/feof> function in the case of Linux.
+
+The file stream is a L<Sys::IO::FileStream|SPVM::Sys::IO::FileStream> object.
+
+=head2 ferror
+
+  static method ferror : int ($stream : Sys::IO::FileStream);
+
+The function ferror() tests the error indicator for the stream pointed to by stream, returning nonzero if it is set. The error indicator can only be reset by the clearerr() function.
+
+See the detail of the L<ferror|https://linux.die.net/man/3/ferror> function in the case of Linux.
+
+The file stream is a L<Sys::IO::FileStream|SPVM::Sys::IO::FileStream> object.
+
+=head2 clearerr
+
+  static method clearerr : void ($stream : Sys::IO::FileStream);
+
+The function clearerr() clears the end-of-file and error indicators for the stream pointed to by stream.
+
+See the detail of the L<clearerr|https://linux.die.net/man/3/clearerr> function in the case of Linux.
 
 The file stream is a L<Sys::IO::FileStream|SPVM::Sys::IO::FileStream> object.
 
@@ -140,12 +140,22 @@ The file stream is a L<Sys::IO::FileStream|SPVM::Sys::IO::FileStream> object.
 
 =head2 fgets
 
-  static method fgets : mutable string ($buffer : mutable string, $size : int, $stream : Sys::IO::FileStream);
+  static method fgets : mutable string ($s : mutable string, $size : int, $stream : Sys::IO::FileStream);
 
 fgets() reads in at most one less than size characters from stream and stores them into the buffer pointed to by s. Reading stops after an EOF or a newline. If a newline is read, it is stored into the buffer. A terminating null byte (aq\0aq) is stored after the last character in the buffer.
 
 See the detail of the L<fgets|https://linux.die.net/man/3/fgets> function in the case of Linux.
 
+=head2 fwrite
+
+  static method fwrite : int ($ptr : string, $size : int, $nmemb : int, $stream : Sys::IO::FileStream);
+
+The function fwrite() writes nmemb elements of data, each size bytes long, to the stream pointed to by stream, obtaining them from the location given by ptr.
+
+See the detail of the L<fread|https://linux.die.net/man/3/fwrite> function in the case of Linux.
+
+The file stream is a L<Sys::IO::FileStream|SPVM::Sys::IO::FileStream> object.
+
 =head2 fseek
 
   static method fseek : int ($stream : Sys::IO::FileStream, $offset : long, $whence : int);
@@ -196,21 +206,79 @@ See the detail of the L<flock|https://linux.die.net/man/2/flock> function in the
 
 See L<Sys::IO::Constant|SPVM::Sys::IO::Constant> about the constant value for the operation.
 
-=head2 read
+=head2 mkdir
 
-  static method read : int ($fd : int, $buffer : mutable string, $count : int);
+  static method mkdir : int ($path : string, $mode : int);
 
-read() attempts to read up to count bytes from file descriptor fd into the buffer starting at buf.
+mkdir() attempts to create a directory named pathname.
 
-See the detail of the L<read|https://linux.die.net/man/2/read> function in the case of Linux.
+See the detail of the L<mkdir|https://linux.die.net/man/2/mkdir> function in the case of Linux.
 
-=head2 write
+See L<Sys::IO::Constant|SPVM::Sys::IO::Constant> about the constant value for the mode.
 
-  static method write : int ($fd : int, $buffer : string, $count : int);
+=head2 umask
 
-write() writes up to count bytes from the buffer pointed buf to the file referred to by the file descriptor fd.
+  static method umask : int ($mode : int);
 
-See the detail of the L<write|https://linux.die.net/man/2/write> function in the case of Linux.
+umask() sets the calling process's file mode creation mask (umask) to mask & 0777 (i.e., only the file permission bits of mask are used), and returns the previous value of the mask.
+
+See the detail of the L<umask|https://linux.die.net/man/2/umask> function in the case of Linux.
+
+=head2 rmdir
+
+  static method rmdir : int ($path : string);
+
+rmdir() deletes a directory, which must be empty.
+
+See the detail of the L<rmdir|https://linux.die.net/man/2/rmdir> function in the case of Linux.
+
+=head2 unlink
+
+  static method unlink : int ($pathname : string);
+
+unlink() deletes a name from the file system. If that name was the last link to a file and no processes have the file open the file is deleted and the space it was using is made available for reuse.
+
+See the detail of the L<unlink|https://linux.die.net/man/2/unlink> function in the case of Linux.
+
+=head2 rename
+
+  static method rename : int ($oldpath : string, $newpath : string);
+
+rename() renames a file, moving it between directories if required. Any other hard links to the file (as created using link(2)) are unaffected. Open file descriptors for oldpath are also unaffected.
+
+See the detail of the L<rename|https://linux.die.net/man/2/rename> function in the case of Linux.
+
+=head2 getcwd
+
+  static method getcwd : mutable string ($buf : mutable string, $size : int);
+
+The getcwd() function copies an absolute pathname of the current working directory to the array pointed to by buf, which is of length size.
+
+See the detail of the L<getcwd|https://linux.die.net/man/2/getcwd> function in the case of Linux.
+
+=head2 _getdcwd
+
+  static method _getdcwd : mutable string ($drive : int, $buffer : mutable string, $maxlen : int);
+
+Gets the full path of the current working directory on the specified drive.
+
+See the detail of the L<_getdcwd|https://docs.microsoft.com/en-us/cpp/c-runtime-library/reference/getdcwd-wgetdcwd?view=msvc-170> function in the case of Windows.
+
+=head2 realpath
+
+  static method realpath : mutable string ($path : string, $resolved_path : mutable string);
+
+realpath() expands all symbolic links and resolves references to /./, /../ and extra '/' characters in the null-terminated string named by path to produce a canonicalized absolute pathname. The resulting pathname is stored as a null-terminated string, up to a maximum of PATH_MAX bytes, in the buffer pointed to by resolved_path. The resulting path will have no symbolic link, /./ or /../ components.
+
+See the detail of the L<realpath|https://linux.die.net/man/3/realpath> function in the case of Linux.
+
+=head2 _fullpath
+
+  native static method _fullpath : mutable string ($absPath : mutable string, $relPath : string, $maxLength : int);
+
+Creates an absolute or full path name for the specified relative path name.
+
+See the detail of the L<_fullpath|https://docs.microsoft.com/en-us/cpp/c-runtime-library/reference/fullpath-wfullpath?view=msvc-170> function in the case of Windows.
 
 =head2 chdir
 
@@ -238,15 +306,29 @@ chown() changes the ownership of the file specified by path, which is dereferenc
 
 See the detail of the L<chown|https://linux.die.net/man/2/chown> function in the case of Linux.
 
-=head2 mkdir
+=head2 truncate
 
-  static method mkdir : int ($path : string, $mode : int);
+  static method truncate : int ($path : string, $length : long);
 
-mkdir() attempts to create a directory named pathname.
+The truncate() and ftruncate() functions cause the regular file named by path or referenced by fd to be truncated to a size of precisely length bytes.
 
-See the detail of the L<mkdir|https://linux.die.net/man/2/mkdir> function in the case of Linux.
+See the detail of the L<truncate|https://linux.die.net/man/2/truncate> function in the case of Linux.
 
-See L<Sys::IO::Constant|SPVM::Sys::IO::Constant> about the constant value for the mode.
+=head2 readlink
+
+  static method readlink : int ($path : string, $buffer : mutable string, $buffer_size : int);
+
+readlink() places the contents of the symbolic link path in the buffer buf, which has size bufsiz. readlink() does not append a null byte to buf. It will truncate the contents (to a length of bufsiz characters), in case the buffer is too small to hold all of the contents.
+
+See the detail of the L<readlink|https://linux.die.net/man/2/readlink> function in the case of Linux.
+
+=head2 symlink
+
+  static method symlink : int ($oldpath : string, $newpath : string);
+
+symlink() creates a symbolic link named newpath which contains the string oldpath.
+
+See the detail of the L<symlink|https://linux.die.net/man/2/symlink> function in the case of Linux.
 
 =head2 opendir
 
@@ -308,59 +390,9 @@ See the detail of the L<seekdir|https://linux.die.net/man/3/seekdir> function in
 
 The directory stream is a L<Sys::IO::DirStream|SPVM::Sys::IO::DirStream> object.
 
-=head2 truncate
-
-  static method truncate : long ($path : string, $offset : long);
-
-The truncate() and ftruncate() functions cause the regular file named by path or referenced by fd to be truncated to a size of precisely length bytes.
-
-See the detail of the L<truncate|https://linux.die.net/man/2/truncate> function in the case of Linux.
-
-=head2 lseek
-
-  static method lseek : long ($fd : int, $offset : long, $whence : int);
-
-The lseek() function repositions the offset of the open file associated with the file descriptor fd to the argument offset according to the directive whence as follows:
-
-See the detail of the L<lseek|https://linux.die.net/man/2/lseek> function in the case of Linux.
-
-See L<Sys::IO::Constant|SPVM::Sys::IO::Constant> about the constant value for the whence.
-
-=head2 umask
-
-  static method umask : int ($mode : int);
-
-umask() sets the calling process's file mode creation mask (umask) to mask & 0777 (i.e., only the file permission bits of mask are used), and returns the previous value of the mask.
-
-See the detail of the L<lseek|https://linux.die.net/man/2/umask> function in the case of Linux.
-
-=head2 rename
-
-  static method rename : int ($old_path : string, $new_path : string);
-
-rename() renames a file, moving it between directories if required. Any other hard links to the file (as created using link(2)) are unaffected. Open file descriptors for oldpath are also unaffected.
-
-See the detail of the L<rename|https://linux.die.net/man/2/rename> function in the case of Linux.
-
-=head2 readlink
-
-  static method readlink : int ($path : string, $buffer : mutable string, $buffer_size : int);
-
-readlink() places the contents of the symbolic link path in the buffer buf, which has size bufsiz. readlink() does not append a null byte to buf. It will truncate the contents (to a length of bufsiz characters), in case the buffer is too small to hold all of the contents.
-
-See the detail of the L<readlink|https://linux.die.net/man/2/readlink> function in the case of Linux.
-
-=head2 symlink
-
-  static method symlink : int ($target : string, $link_path : string);
-
-symlink() creates a symbolic link named newpath which contains the string oldpath.
-
-See the detail of the L<symlink|https://linux.die.net/man/2/symlink> function in the case of Linux.
-
 =head2 utime
 
-  static method utime : int ($file : string, $buffer : Sys::IO::Utimbuf);
+  static method utime : int ($filename : string, $times : Sys::IO::Utimbuf);
 
 The utime() system call changes the access and modification times of the inode specified by filename to the actime and modtime fields of times respectively.
 
